﻿using System.Collections.Generic;
using System.Linq;
using Sdl.FileTypeSupport.Framework.BilingualApi;
using Sdl.ProjectAutomation.Core;

namespace CustomWordCounter.Content
{


    public class MyWordCountInfo
    {
        public int Segments { get; set; }
        public string WordCountType { get; set; }
        public int Words { get; set; }
        public int Characters { get; set; }
        public int Placeables { get; set; }
        public int Tags { get; set; }
    }



    public class ContentProcessor : IBilingualContentProcessor
    {

        private ProjectFile ProjectFile { get; set; }
        public IDocumentProperties DocumentProperties { get; set; }
        private IFileProperties FileProperties { get; set; }
        public Dictionary<string, List<MyWordCountInfo>> MyWordCounts { get; set; }

        public Dictionary<string, List<ISegmentPair>> IsRepeatedSegments { get; set; }

        private ContentReader _contentReader;
        public ContentReader MyContentReader => _contentReader ?? (_contentReader = new ContentReader());

        private TemporaryAnalysisTm _temporaryAnalysisTm;
        public TemporaryAnalysisTm TemporaryAnalysisTm => _temporaryAnalysisTm ?? (_temporaryAnalysisTm = new TemporaryAnalysisTm());



        public ContentProcessor(ProjectFile projectFile)
        {
            ProjectFile = projectFile;
        }
        public IBilingualContentHandler Output
        {
            get;
            set;
        }

        public void Complete()
        {
            // not needed for this implementation
        }

        public void FileComplete()
        {
            // not needed for this implementation
        }

        public void Initialize(IDocumentProperties documentInfo)
        {
            
            DocumentProperties = documentInfo;

            MyWordCounts = new Dictionary<string, List<MyWordCountInfo>>();
            IsRepeatedSegments = new Dictionary<string, List<ISegmentPair>>();

            // create a temporary tm for performing an analysis
            TemporaryAnalysisTm.CreateTranslationMemory(documentInfo);
        }

        public void ProcessParagraphUnit(IParagraphUnit paragraphUnit)
        {
            foreach (var segmentPair in paragraphUnit.SegmentPairs)
            {
                if (segmentPair.Properties.TranslationOrigin == null)
                    continue;

                if (segmentPair.Properties.TranslationOrigin.IsRepeated)
                    IsRepeatedSegments[FileProperties.FileConversionProperties.InputFilePath].Add(segmentPair);

                MyContentReader.ProcessSegment(segmentPair.Source);
                MyContentReader.Segment.Culture = DocumentProperties.SourceLanguage.CultureInfo;

                // you can also recover the word count information by searching against a tempoary TM                
                var searchResults = TemporaryAnalysisTm.SearchTranslationMemory(MyContentReader.Segment);
                var wordCounts = searchResults.SourceWordCounts;

                //var wordCounts = new WordCounts(MyContentReader.Segment.Tokens, true, true, true, true);
                var wordCountType = Helper.GetOriginType(segmentPair.Properties.TranslationOrigin);

                var fileWordCounts = MyWordCounts[FileProperties.FileConversionProperties.InputFilePath];
                var wordCountTypeInfo = fileWordCounts.FirstOrDefault(a => a.WordCountType == wordCountType);
                if (wordCountTypeInfo == null)
                {
                    MyWordCounts[FileProperties.FileConversionProperties.InputFilePath].Add(new MyWordCountInfo
                    {
                        Segments = 1,
                        WordCountType = wordCountType,
                        Words = wordCounts.Words,
                        Characters = wordCounts.Characters,
                        Placeables = wordCounts.Placeables,
                        Tags = wordCounts.Tags
                    });
                }
                else
                {
                    wordCountTypeInfo.Segments++;
                    wordCountTypeInfo.Words += wordCounts.Words;
                    wordCountTypeInfo.Characters += wordCounts.Characters;
                    wordCountTypeInfo.Placeables += wordCounts.Placeables;
                    wordCountTypeInfo.Tags += wordCounts.Tags;
                }
            }
        }


        public void SetFileProperties(IFileProperties fileInfo)
        {
            FileProperties = fileInfo;
            MyWordCounts.Add(FileProperties.FileConversionProperties.InputFilePath, new List<MyWordCountInfo>());
            IsRepeatedSegments.Add(FileProperties.FileConversionProperties.InputFilePath, new List<ISegmentPair>());
        }
    }
}
