﻿using Sdl.ProjectAutomation.AutomaticTasks;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using CustomWordCounter.Content;
using Sdl.FileTypeSupport.Framework.IntegrationApi;
using Sdl.ProjectAutomation.Core;

namespace CustomWordCounter
{
    [AutomaticTask("CustomWordCounter101",
        "Custom Word Counter",
        "Custom Word Counter",
        //[TODO] You can change the file type according to your needs
        GeneratedFileType = AutomaticTaskFileType.BilingualTarget)]
    //[TODO] You can change the file type according to your needs
    [AutomaticTaskSupportedFileType(AutomaticTaskFileType.BilingualTarget)]
    [RequiresSettings(typeof(MyCustomBatchTaskSettings), typeof(MyCustomBatchTaskSettingsPage))]
    public class MyCustomBatchTask : AbstractFileContentProcessingAutomaticTask
    {
       

        protected override void ConfigureConverter(ProjectFile projectFile, IMultiFileConverter multiFileConverter)
        {
            
            var contentProcessor = new ContentProcessor(projectFile);
            multiFileConverter.AddBilingualProcessor(contentProcessor);
            multiFileConverter.Parse();
            ViewAnalysisReport(contentProcessor);
        }

        private void ViewAnalysisReport(ContentProcessor contentProcessor)
        {
            var reportPath = Path.Combine(CustomPathInfo.MyCustomReportPath, "Report." + DateTimeStamp() + ".txt");

            using (var sw = new StreamWriter(reportPath, false, Encoding.UTF8))
            {
                foreach (var myWordCount in contentProcessor.MyWordCounts)
                {
                    sw.WriteLine("File: {0}", myWordCount.Key);

                    sw.WriteLine();
                    sw.WriteLine("Repetitions");
                    foreach (var isRepeatedSegment in contentProcessor.IsRepeatedSegments[myWordCount.Key])
                    {
                        sw.WriteLine("tu=\"{0}\"\tseg=\"{1}\"", isRepeatedSegment.Source.ParentParagraphUnit.Properties.ParagraphUnitId.Id, isRepeatedSegment.Properties.Id);
                    }



                    var totalSegments = 0;
                    var totalWords = 0;
                    var totalCharacters = 0;
                    var totalTags = 0;
                    //var totalPlaceables = 0;


                    foreach (var wordCount in myWordCount.Value)
                    {
                        totalSegments += wordCount.Segments;
                        totalWords += wordCount.Words;
                        totalCharacters += wordCount.Characters;
                        totalTags += wordCount.Tags;
                        //totalPlaceables += wordCount.Placeables;
                    }

                    sw.WriteLine();
                    sw.WriteLine("Analysis");
                    sw.WriteLine("{0}\t{1}\t{2}\t{3}\t{4}\t{5}", "Type         ", "Segs", "Words", "Chars", "Percent", "Tags");
                    WriteAnalysisBand(sw, myWordCount.Value, totalWords, "PerfectMatch ");
                    WriteAnalysisBand(sw, myWordCount.Value, totalWords, "Context Match");
                    WriteAnalysisBand(sw, myWordCount.Value, totalWords, "Repetition   ");
                    WriteAnalysisBand(sw, myWordCount.Value, totalWords, "100%         ");
                    WriteAnalysisBand(sw, myWordCount.Value, totalWords, "95% - 99%    ");
                    WriteAnalysisBand(sw, myWordCount.Value, totalWords, "85% - 94%    ");
                    WriteAnalysisBand(sw, myWordCount.Value, totalWords, "75% - 84%    ");
                    WriteAnalysisBand(sw, myWordCount.Value, totalWords, "50% - 74%    ");
                    WriteAnalysisBand(sw, myWordCount.Value, totalWords, "New          ");
                    WriteAnalysisBand(sw, myWordCount.Value, totalWords, "AdaptiveMT   ");
                    WriteAnalysisBand(sw, myWordCount.Value, totalWords, "AT           ");
                    sw.WriteLine("{0}\t{1}\t{2}\t{3}\t{4}\t{5}", "Total        ", totalSegments, totalWords, totalCharacters,
                        "100%", totalTags);
                    sw.WriteLine();
                }
            }


            Process.Start(reportPath);
        }

        private CustomPaths _customPathInfo;
        private CustomPaths CustomPathInfo => _customPathInfo ?? (_customPathInfo = new CustomPaths());

        private static string DateTimeStamp()
        {
            var dts = DateTime.Now;
            return dts.Year
                   + "" + dts.Month.ToString().PadLeft(2, '0')
                   + "" + dts.Day.ToString().PadLeft(2, '0')
                   + "" + dts.Hour.ToString().PadLeft(2, '0')
                   + "" + dts.Minute.ToString().PadLeft(2, '0')
                   + "" + dts.Second.ToString().PadLeft(2, '0');
        }

        private static MyWordCountInfo WordCountTypeInfo(IEnumerable<MyWordCountInfo> myWordCount, string wordCountType)
        {
            return myWordCount
                       .FirstOrDefault(a => wordCountType == a.WordCountType) ??
                   new MyWordCountInfo
                   {
                       Segments = 0,
                       WordCountType = wordCountType,
                       Words = 0,
                       Characters = 0,
                       Placeables = 0,
                       Tags = 0
                   };
        }

        private static decimal WordCountPercentage(decimal total, decimal quantity)
        {
            if (total == 0 || quantity == 0 || quantity > total)
                return 0;

            var percentage = Math.Round(quantity / total * 100, 0);

            return percentage;
        }

        private static void WriteAnalysisBand(TextWriter sw, IEnumerable<MyWordCountInfo> myWordCount, int totalWords, string wordCountType)
        {
            var analysisWordCount = WordCountTypeInfo(myWordCount, wordCountType.Trim());
            sw.WriteLine("{0}\t{1}\t{2}\t{3}\t{4}\t{5}", wordCountType, analysisWordCount.Segments, analysisWordCount.Words,
                analysisWordCount.Characters, WordCountPercentage(totalWords, analysisWordCount.Words), analysisWordCount.Tags);
        }
    }
}
